@@ -1,8 +1,8 @@
 #########################################################################################
 # Package       HiPi::BCM2835
-# Description:  Wrapper for bcm2835 C library
+# Description:  Wrapper for bcm2835 C library - Access to /dev/mem
 # Created       Fri Nov 23 13:55:49 2012
-# SVN Id        $Id: BCM2835.pm 446 2013-02-01 02:55:27Z Mark Dootson $
+# SVN Id        $Id: BCM2835.pm 1026 2013-03-11 08:55:02Z Mark Dootson $
 # Copyright:    Copyright (c) 2012 Mark Dootson
 # Licence:      This work is free software; you can redistribute it and/or modify it 
 #               under the terms of the GNU General Public License as published by the 
@@ -14,307 +14,52 @@ package HiPi::BCM2835;
 
 #########################################################################################
 
-=head1 NAME
-
-HiPi::BCM2835 
-
-=head1 VERSION
-
-Version 0.01
-
-=head1 SYNOPSYS
-
-    use HiPi::BCM2835 qw( :registers :memory :function
-                          :pud :pad :pins :spi :pwm);
-
-    HiPi::BCM2835::bcm2835_init();
-    ...
-    ...
-    HiPi::BCM2835::bcm2835_close();
-    
-=head1 DESCRIPTION
-
-This module is a thin wrapper around the excellent BCM2835 C library by
-Mike McCauley, C<<mikem@open.com.au>>
-
-The original library and documentation  are available at:
-
-L<http://www.open.com.au/mikem/bcm2835/>
-
-Mike also provides his own wrapper - the CPAN module Device::BCM2835
-
-Normally this module is not used directly in end user code. The modules
-HiPi::GPIO, HiPi::GPIO::PAD1 and HiPi::GPIO::PAD5 are intended for
-direct end user use.
-
-=head1 EXPORTED CONSTANT TAGS
-
-=head2 :registers
-
-    BCM2835_PERI_BASE
-    BCM2835_GPIO_PADS
-    BCM2835_CLOCK_BASE
-    BCM2835_GPIO_BASE
-    BCM2835_SPI0_BASE
-    BCM2835_GPIO_PWM
-
-=head2 :memory
-
-    BCM2835_PAGE_SIZE
-    BCM2835_BLOCK_SIZE
-
-=head2 :function
-
-    BCM2835_GPFSEL0
-    BCM2835_GPFSEL1
-    BCM2835_GPFSEL2
-    BCM2835_GPFSEL3
-    BCM2835_GPFSEL4
-    BCM2835_GPFSEL5
-    BCM2835_GPSET0
-    BCM2835_GPSET1
-    BCM2835_GPCLR0
-    BCM2835_GPCLR1
-    BCM2835_GPLEV0
-    BCM2835_GPLEV1
-    BCM2835_GPEDS0
-    BCM2835_GPEDS1
-    BCM2835_GPREN0
-    BCM2835_GPREN1
-    BCM2835_GPFEN0
-    BCM2835_GPFEN1
-    BCM2835_GPHEN0
-    BCM2835_GPHEN1
-    BCM2835_GPLEN0
-    BCM2835_GPLEN1
-    BCM2835_GPAREN0
-    BCM2835_GPAREN1
-    BCM2835_GPAFEN0
-    BCM2835_GPAFEN1
-    BCM2835_GPPUD
-    BCM2835_GPPUDCLK0
-    BCM2835_GPPUDCLK1
-    BCM2835_GPIO_FSEL_INPT
-    BCM2835_GPIO_FSEL_OUTP
-    BCM2835_GPIO_FSEL_ALT0
-    BCM2835_GPIO_FSEL_ALT1
-    BCM2835_GPIO_FSEL_ALT2
-    BCM2835_GPIO_FSEL_ALT3
-    BCM2835_GPIO_FSEL_ALT4
-    BCM2835_GPIO_FSEL_ALT5
-    BCM2835_GPIO_FSEL_MASK
-
-=head2 :pud
-
-    BCM2835_GPIO_PUD_OFF
-    BCM2835_GPIO_PUD_DOWN
-    BCM2835_GPIO_PUD_UP
-
-=head2 :pads
-
-    BCM2835_PADS_GPIO_0_27  
-    BCM2835_PADS_GPIO_28_45
-    BCM2835_PADS_GPIO_46_53  
-    BCM2835_PAD_PASSWRD   
-    BCM2835_PAD_SLEW_RATE_UNLIMITED
-    BCM2835_PAD_HYSTERESIS_ENABLED 
-    BCM2835_PAD_DRIVE_2mA 
-    BCM2835_PAD_DRIVE_4mA 
-    BCM2835_PAD_DRIVE_6mA 
-    BCM2835_PAD_DRIVE_8mA 
-    BCM2835_PAD_DRIVE_10mA 
-    BCM2835_PAD_DRIVE_12mA 
-    BCM2835_PAD_DRIVE_14mA 
-    BCM2835_PAD_DRIVE_16mA 
-    BCM2835_PAD_GROUP_GPIO_0_27 
-    BCM2835_PAD_GROUP_GPIO_28_45 
-    BCM2835_PAD_GROUP_GPIO_46_53 
-
-=head2 :pins
-
-    RPI_GPIO_P1_03  
-    RPI_GPIO_P1_05
-    RPI_GPIO_P1_07
-    RPI_GPIO_P1_08
-    RPI_GPIO_P1_10
-    RPI_GPIO_P1_11 
-    RPI_GPIO_P1_12  
-    RPI_GPIO_P1_13  
-    RPI_GPIO_P1_15   
-    RPI_GPIO_P1_16 
-    RPI_GPIO_P1_18 
-    RPI_GPIO_P1_19 
-    RPI_GPIO_P1_21 
-    RPI_GPIO_P1_22 
-    RPI_GPIO_P1_23  
-    RPI_GPIO_P1_24 
-    RPI_GPIO_P1_26      
-    RPI_V2_GPIO_P1_03  
-    RPI_V2_GPIO_P1_05  
-    RPI_V2_GPIO_P1_07  
-    RPI_V2_GPIO_P1_08 
-    RPI_V2_GPIO_P1_10
-    RPI_V2_GPIO_P1_11 
-    RPI_V2_GPIO_P1_12 
-    RPI_V2_GPIO_P1_13 
-    RPI_V2_GPIO_P1_15 
-    RPI_V2_GPIO_P1_16 
-    RPI_V2_GPIO_P1_18 
-    RPI_V2_GPIO_P1_19 
-    RPI_V2_GPIO_P1_21 
-    RPI_V2_GPIO_P1_22 
-    RPI_V2_GPIO_P1_23 
-    RPI_V2_GPIO_P1_24 
-    RPI_V2_GPIO_P1_26
-    RPI_V2_GPIO_P5_03
-    RPI_V2_GPIO_P5_04
-    RPI_V2_GPIO_P5_05
-    RPI_V2_GPIO_P5_06   
-
-=head2 :spi
-
-    BCM2835_SPI0_CS  
-    BCM2835_SPI0_FIFO
-    BCM2835_SPI0_CLK 
-    BCM2835_SPI0_DLEN  
-    BCM2835_SPI0_LTOH  
-    BCM2835_SPI0_DC  
-    BCM2835_SPI0_CS_LEN_LONG
-    BCM2835_SPI0_CS_DMA_LEN
-    BCM2835_SPI0_CS_CSPOL2 
-    BCM2835_SPI0_CS_CSPOL1
-    BCM2835_SPI0_CS_CSPOL0 
-    BCM2835_SPI0_CS_RXF  
-    BCM2835_SPI0_CS_RXR
-    BCM2835_SPI0_CS_TXD 
-    BCM2835_SPI0_CS_RXD 
-    BCM2835_SPI0_CS_DONE 
-    BCM2835_SPI0_CS_TE_EN 
-    BCM2835_SPI0_CS_LMONO 
-    BCM2835_SPI0_CS_LEN
-    BCM2835_SPI0_CS_REN 
-    BCM2835_SPI0_CS_ADCS 
-    BCM2835_SPI0_CS_INTR 
-    BCM2835_SPI0_CS_INTD
-    BCM2835_SPI0_CS_DMAEN 
-    BCM2835_SPI0_CS_TA 
-    BCM2835_SPI0_CS_CSPOL 
-    BCM2835_SPI0_CS_CLEAR 
-    BCM2835_SPI0_CS_CLEAR_RX 
-    BCM2835_SPI0_CS_CLEAR_TX
-    BCM2835_SPI0_CS_CPOL 
-    BCM2835_SPI0_CS_CPHA 
-    BCM2835_SPI0_CS_CS 
-    BCM2835_SPI_BIT_ORDER_LSBFIRST
-    BCM2835_SPI_BIT_ORDER_MSBFIRST
-    BCM2835_SPI_MODE0 
-    BCM2835_SPI_MODE1
-    BCM2835_SPI_MODE2
-    BCM2835_SPI_MODE3 
-    BCM2835_SPI_CS0 
-    BCM2835_SPI_CS1 
-    BCM2835_SPI_CS2 
-    BCM2835_SPI_CS_NONE 
-    BCM2835_SPI_CLOCK_DIVIDER_65536
-    BCM2835_SPI_CLOCK_DIVIDER_32768 
-    BCM2835_SPI_CLOCK_DIVIDER_16384
-    BCM2835_SPI_CLOCK_DIVIDER_8192
-    BCM2835_SPI_CLOCK_DIVIDER_4096 
-    BCM2835_SPI_CLOCK_DIVIDER_2048 
-    BCM2835_SPI_CLOCK_DIVIDER_1024
-    BCM2835_SPI_CLOCK_DIVIDER_512 
-    BCM2835_SPI_CLOCK_DIVIDER_256
-    BCM2835_SPI_CLOCK_DIVIDER_128 
-    BCM2835_SPI_CLOCK_DIVIDER_64
-    BCM2835_SPI_CLOCK_DIVIDER_32 
-    BCM2835_SPI_CLOCK_DIVIDER_16
-    BCM2835_SPI_CLOCK_DIVIDER_8
-    BCM2835_SPI_CLOCK_DIVIDER_4
-    BCM2835_SPI_CLOCK_DIVIDER_2
-    BCM2835_SPI_CLOCK_DIVIDER_1
-
-=head2 :pwm
-
-    BCM2835_PWM_CONTROL
-    BCM2835_PWM_STATUS
-    BCM2835_PWM0_RANGE 
-    BCM2835_PWM0_DATA
-    BCM2835_PWM1_RANGE 
-    BCM2835_PWM1_DATA 
-    BCM2835_PWMCLK_CNTL
-    BCM2835_PWMCLK_DIV 
-    BCM2835_PWM1_MS_MODE
-    BCM2835_PWM1_USEFIFO
-    BCM2835_PWM1_REVPOLAR
-    BCM2835_PWM1_OFFSTATE
-    BCM2835_PWM1_REPEATFF
-    BCM2835_PWM1_SERIAL
-    BCM2835_PWM1_ENABLE
-    BCM2835_PWM0_MS_MODE
-    BCM2835_PWM0_USEFIFO
-    BCM2835_PWM0_REVPOLAR
-    BCM2835_PWM0_OFFSTATE
-    BCM2835_PWM0_REPEATFF
-    BCM2835_PWM0_SERIAL
-    BCM2835_PWM0_ENABLE
-
-=head1 METHODS
-
-The methods listed at L<http://www.open.com.au/mikem/bcm2835/modules.html>
-are wrapped.
-
-An example to set Raspberry Pi GPIO Pin 11 mode to output and 'on', then
-reset RPi pins 3 & 5 for standard I2C use (alt function 0);
-
-    use HiPi::BCM2835 qw( :pins :function );
-    HiPi::BCM2835::bcm2835_gpio_fsel( RPI_V2_GPIO_P1_11 , BCM2835_GPIO_FSEL_OUTP);
-    HiPi::BCM2835::bcm2835_gpio_set( RPI_V2_GPIO_P1_11);
-    
-    HiPi::BCM2835::bcm2835_gpio_fsel( RPI_V2_GPIO_P1_03 , BCM2835_GPIO_FSEL_ALT0);
-    HiPi::BCM2835::bcm2835_gpio_fsel( RPI_V2_GPIO_P1_05 , BCM2835_GPIO_FSEL_ALT0);
-
-Note that the HiPi::BCM2835 functions expect pin numbers to be the Broadcomm
-BCM 2835 GPIO pin numbers - which are different to the Raspberry Pi Pad 1 pin
-header layout. Use the constants RPI_V2_GPIO_P1_xx to get BCM GPIO numbers for
-Raspberry Pi GPIO header pin numbers.
-
-=head1 LICENSE
-
-This work is free software; you can redistribute it and/or modify it 
-under the terms of the GNU General Public License as published by the 
-Free Software Foundation; either version 3 of the License, or any later 
-version.
-
-=head2 License Note
-
-I would normally release any Perl code under the Perl Artistic License
-but this module wraps several GPL / LGPL C libraries and I feel that
-the licensing of the entire distribution is simpler if the Perl code
-is under GPL too.
-
-=head1 AUTHOR
-
-Mark Dootson, C<< <mdootson at cpan.org> >>
-
-=head1 COPYRIGHT
-
-Copyright (C) 2012-2013 Mark Dootson, all rights reserved.
-
-=cut
-
+use 5.14.0;
 use strict;
 use warnings;
-use Exporter;
-use base qw( Exporter );
+use parent qw( HiPi::Device );
 use XSLoader;
+use Carp;
+use HiPi;
+use HiPi::Utils qw( is_raspberry );
+use HiPi::Constant qw( :raspberry :spi :i2c);
 
-our $VERSION = '0.01';
+our $VERSION = '0.20';
 
-unless($ENV{DEV_NONE_RASPBERRY_PLATFORM}) {
-    # on real Raspberry Pi
+if( is_raspberry ) {
     XSLoader::load('HiPi::BCM2835', $VERSION);
+} else {
+    require HiPi::Dummy::BCM2835;
 }
 
+our $_memmapped = 0;
+
+sub bcm2835_init {
+    unless( $_memmapped ) {
+        _hipi_bcm2835_init() or croak 'Failed to initialise libbcm2835';
+        $_memmapped = 1;
+    }
+}
+
+sub bcm2835_close {
+    if( $_memmapped ) {
+        _hipi_bcm2835_close();
+        $_memmapped = 0;
+    }
+}
+
+
+END{
+    bcm2835_close();
+}
+
+sub CLONE_SKIP {
+    return 1;
+}
+
+
+require HiPi::BCM2835::Pin;
+
 # Constants for libbcm2835
 
 our @EXPORT_OK = ();
@@ -326,29 +71,46 @@ sub _register_exported_constants {
     push( @EXPORT_OK, @constants);
 }
 
-# define HIGH / LOW at HiPi level as RPI_HIGH / RPI_LOW
+# define HIGH / LOW at MyPi level as RPI_HIGH / RPI_LOW
+
+use constant {
+    BCM2835_CORE_CLK_HZ => 250000000,
+    BCM2835_ST_CS       =>    0x0000,
+    BCM2835_ST_CLO      =>    0x0004,
+    BCM2835_ST_CHI      =>    0x0008,
+};
+
+_register_exported_constants( qw(
+    clock
+    BCM2835_CORE_CLK_HZ
+    BCM2835_ST_CS 
+    BCM2835_ST_CLO
+    BCM2835_ST_CHI
+    ) );
 
 #-------------------------------------------------------
 # Physical addresses for various peripheral regiser sets
 #-------------------------------------------------------
 
-use constant {
-    BCM2835_PERI_BASE   => 0x20000000,
-    BCM2835_GPIO_PADS   => 0x20000000 + 0x100000,
-    BCM2835_CLOCK_BASE  => 0x20000000 + 0x101000,
-    BCM2835_GPIO_BASE   => 0x20000000 + 0x200000,
-    BCM2835_SPI0_BASE   => 0x20000000 + 0x204000,
-    BCM2835_GPIO_PWM    => 0x20000000 + 0x20C000,
-};
+sub BCM2835_GPIO_BASE  { bcm2835_gpio(); }
+sub BCM2835_GPIO_PWM   { bcm2835_pwm();  }
+sub BCM2835_CLOCK_BASE { bcm2835_clk();  }
+sub BCM2835_GPIO_PADS  { bcm2835_pads(); }
+sub BCM2835_SPI0_BASE  { bcm2835_spi0(); }
+sub BCM2835_BSC0_BASE  { bcm2835_bsc0(); }
+sub BCM2835_BSC1_BASE  { bcm2835_bsc1(); }
+sub BCM2835_ST_BASE    { bcm2835_st();   }
 
 _register_exported_constants( qw(
     registers
-    BCM2835_PERI_BASE
+    BCM2835_ST_BASE
     BCM2835_GPIO_PADS
     BCM2835_CLOCK_BASE
     BCM2835_GPIO_BASE
     BCM2835_SPI0_BASE
+    BCM2835_BSC0_BASE
     BCM2835_GPIO_PWM
+    BCM2835_BSC1_BASE
     ) );
 
 #-------------------------------------------------------
@@ -390,7 +152,7 @@ use constant {
     BCM2835_GPEDS1     => 0x0044, # GPIO Pin Event Detect Status 1
     BCM2835_GPREN0     => 0x004c, # GPIO Pin Rising Edge Detect Enable 0
     BCM2835_GPREN1     => 0x0050, # GPIO Pin Rising Edge Detect Enable 1
-    BCM2835_GPFEN0     => 0x0048, # GPIO Pin Falling Edge Detect Enable 0
+    BCM2835_GPFEN0     => 0x0058, # GPIO Pin Falling Edge Detect Enable 0
     BCM2835_GPFEN1     => 0x005c, # GPIO Pin Falling Edge Detect Enable 1
     BCM2835_GPHEN0     => 0x0064, # GPIO Pin High Detect Enable 0
     BCM2835_GPHEN1     => 0x0068, # GPIO Pin High Detect Enable 1
@@ -798,7 +560,6 @@ use constant {
     BCM2835_PWM0_ENABLE    => 0x0001, # Channel Enable
 };
 
-
 _register_exported_constants( qw(
     pwm
     BCM2835_PWM_CONTROL
@@ -825,6 +586,332 @@ _register_exported_constants( qw(
     BCM2835_PWM0_ENABLE
     ) );
 
+#-------------------------------------------------------------
+# Defines for I2C
+#-------------------------------------------------------------
 
+use constant {
+    BCM2835_BSC_C                    => 0x0000,      # BSC Master Control
+    BCM2835_BSC_S 		     => 0x0004,      # BSC Master Status
+    BCM2835_BSC_DLEN		     => 0x0008,      # BSC Master Data Length
+    BCM2835_BSC_A 		     => 0x000c,      # BSC Master Slave Address
+    BCM2835_BSC_FIFO		     => 0x0010,      # BSC Master Data FIFO
+    BCM2835_BSC_DIV		     => 0x0014,      # BSC Master Clock Divider
+    BCM2835_BSC_DEL		     => 0x0018,      # BSC Master Data Delay
+    BCM2835_BSC_CLKT		     => 0x001c,      # BSC Master Clock Stretch Timeout
+    BCM2835_BSC_C_I2CEN 	     => 0x00008000,  # I2C Enable, 0 = disabled, 1 = enabled
+    BCM2835_BSC_C_INTR 		     => 0x00000400,  # Interrupt on RX
+    BCM2835_BSC_C_INTT 		     => 0x00000200,  # Interrupt on TX
+    BCM2835_BSC_C_INTD 		     => 0x00000100,  # Interrupt on DONE
+    BCM2835_BSC_C_ST 		     => 0x00000080,  # Start transfer, 1 = Start a new transfer
+    BCM2835_BSC_C_CLEAR_1 	     => 0x00000020,  # Clear FIFO Clear
+    BCM2835_BSC_C_CLEAR_2 	     => 0x00000010,  # Clear FIFO Clear
+    BCM2835_BSC_C_READ 		     => 0x00000001,  #	Read transfer
+    BCM2835_BSC_S_CLKT 		     => 0x00000200,  # Clock stretch timeout
+    BCM2835_BSC_S_ERR 		     => 0x00000100,  # ACK error
+    BCM2835_BSC_S_RXF 		     => 0x00000080,  # RXF FIFO full, 0 = FIFO is not full, 1 = FIFO is full
+    BCM2835_BSC_S_TXE 		     => 0x00000040,  # TXE FIFO full, 0 = FIFO is not full, 1 = FIFO is full
+    BCM2835_BSC_S_RXD 		     => 0x00000020,  # RXD FIFO contains data
+    BCM2835_BSC_S_TXD 		     => 0x00000010,  # TXD FIFO can accept data
+    BCM2835_BSC_S_RXR 		     => 0x00000008,  # RXR FIFO needs reading (full)
+    BCM2835_BSC_S_TXW 		     => 0x00000004,  # TXW FIFO needs writing (full)
+    BCM2835_BSC_S_DONE 		     => 0x00000002,  # Transfer DONE
+    BCM2835_BSC_S_TA 		     => 0x00000001,  # Transfer Active
+    BCM2835_BSC_FIFO_SIZE   	     =>	16,          # BSC FIFO size
+    
+    BCM2835_I2C_CLOCK_DIVIDER_2500   => 2500,      # 2500 = 10us = 100 kHz
+    BCM2835_I2C_CLOCK_DIVIDER_626    => 626,       # 622 = 2.504us = 399.3610 kHz
+    BCM2835_I2C_CLOCK_DIVIDER_150    => 150,       # 150 = 60ns = 1.666 MHz (default at reset)
+    BCM2835_I2C_CLOCK_DIVIDER_148    => 148,       # 148 = 59ns = 1.689 MHz
+    BCM2835_I2C_REASON_OK   	     => 0x00,      # Success
+    BCM2835_I2C_REASON_ERROR_NACK    => 0x01,      # Received a NACK
+    BCM2835_I2C_REASON_ERROR_CLKT    => 0x02,      # Received Clock Stretch Timeout
+    BCM2835_I2C_REASON_ERROR_DATA    => 0x04,      # Not all data is sent / received
+};
+
+_register_exported_constants( qw(
+    i2c
+    BCM2835_BSC_C 
+    BCM2835_BSC_S 
+    BCM2835_BSC_DLEN
+    BCM2835_BSC_A 
+    BCM2835_BSC_FIFO
+    BCM2835_BSC_DIV
+    BCM2835_BSC_DEL
+    BCM2835_BSC_CLKT
+    BCM2835_BSC_C_I2CEN 
+    BCM2835_BSC_C_INTR 
+    BCM2835_BSC_C_INTT 
+    BCM2835_BSC_C_INTD 
+    BCM2835_BSC_C_ST 
+    BCM2835_BSC_C_CLEAR_1
+    BCM2835_BSC_C_CLEAR_2
+    BCM2835_BSC_C_READ 
+    BCM2835_BSC_S_CLKT
+    BCM2835_BSC_S_ERR 
+    BCM2835_BSC_S_RXF 
+    BCM2835_BSC_S_TXE 
+    BCM2835_BSC_S_RXD 
+    BCM2835_BSC_S_TXD 
+    BCM2835_BSC_S_RXR 
+    BCM2835_BSC_S_TXW 
+    BCM2835_BSC_S_DONE 
+    BCM2835_BSC_S_TA 
+    BCM2835_BSC_FIFO_SIZE
+    BCM2835_I2C_CLOCK_DIVIDER_2500
+    BCM2835_I2C_CLOCK_DIVIDER_626
+    BCM2835_I2C_CLOCK_DIVIDER_150
+    BCM2835_I2C_CLOCK_DIVIDER_148
+    BCM2835_I2C_REASON_OK
+    BCM2835_I2C_REASON_ERROR_NACK
+    BCM2835_I2C_REASON_ERROR_CLKT
+    BCM2835_I2C_REASON_ERROR_DATA
+    ) );
+
+#-----------------------------------------
+# create OO method wrappers
+#-----------------------------------------
+
+{
+    for my $method( qw(
+        gpio_fget gpio_get_eds get_pin gpio_fget_name
+        set_SPI0 set_I2C0 set_I2C1 set_UART0 set_UART1
+        set_CTS0 set_CTS1 set_PWM0
+        spi_transfern spi_transfernb spi_writenb
+    ) ) {
+        no strict 'refs';
+        my $subkey  = __PACKAGE__  . qq(::$method);
+        my $funckey = qq(hipi_$method);
+        *{$subkey} = sub { shift; &$funckey( @_ ); };
+    }
+    
+    for my $method( qw(
+        peri_read peri_read_nb peri_write peri_write_nb peri_set_bits
+        gpio_fsel gpio_set gpio_clr gpio_set_multi gpio_clr_multi
+        gpio_lev gpio_eds gpio_set_eds gpio_ren gpio_clr_ren
+        gpio_fen gpio_clr_fen gpio_hen gpio_clr_hen gpio_len
+        gpio_clr_len gpio_aren gpio_clr_aren gpio_afen gpio_clr_afen
+        gpio_pud gpio_pudclk gpio_pad gpio_set_pad delay
+        delayMicroseconds gpio_write gpio_write_multi gpio_write_mask
+        gpio_set_pud spi_begin spi_end spi_setBitOrder spi_setClockDivider
+        spi_setDataMode spi_chipSelect spi_setChipSelectPolarity spi_transfer
+        i2c_begin i2c_end
+        i2c_setSlaveAddress i2c_setClockDivider i2c_write i2c_read
+        st_read st_delay
+    ) ) {
+        no strict 'refs';
+        my $subkey  = __PACKAGE__  . qq(::$method);
+        my $funckey = qq(bcm2835_$method);
+        *{$subkey} = sub { shift; &$funckey( @_ ); };
+        
+    }
+}
+
+our @_altnames = (
+    [qw( I2C0_SDA      SA5          ALT2   ALT3   ALT4        ALT5 ) ], # GPIO 0
+    [qw( I2C0_SCL      SA4          ALT2   ALT3   ALT4        ALT5 ) ], # GPIO 1
+    [qw( I2C1_SDA      SA3          ALT2   ALT3   ALT4        ALT5 ) ], # GPIO 2
+    [qw( I2C1_SCL      SA2          ALT2   ALT3   ALT4        ALT5 ) ], # GPIO 3
+    
+    [qw( GPCLK0        SA1          ALT2   ALT3   ALT4        ARM_TDI ) ], # GPIO 4
+    [qw( ALT0          ALT1         ALT2   ALT3   ALT4        ALT5 ) ], # GPIO 5
+    [qw( ALT0          ALT1         ALT2   ALT3   ALT4        ALT5 ) ], # GPIO 6
+    [qw( SPI0_CE1_N    SWE_N/SRW_N  ALT2   ALT3   ALT4        ALT5 ) ], # GPIO 7
+    
+    [qw( SPI0_CE0_N    SD0          ALT2   ALT3   ALT4        ALT5 ) ], # GPIO 8
+    [qw( SPI0_MISO     SD1          ALT2   ALT3   ALT4        ALT5 ) ], # GPIO 9
+    [qw( SPI0_MOSI     SD2          ALT2   ALT3   ALT4        ALT5 ) ], # GPIO 10
+    [qw( SPI0_SCLK     SD3          ALT2   ALT3   ALT4        ALT5 ) ], # GPIO 11
+    
+    [qw( ALT0          ALT1         ALT2   ALT3   ALT4        ALT5 ) ], # GPIO 12
+    [qw( ALT0          ALT1         ALT2   ALT3   ALT4        ALT5 ) ], # GPIO 13
+    [qw( UART0_TXD     SD6          ALT2   ALT3   ALT4   UART1_TXD ) ], # GPIO 14
+    [qw( UART0_RXD     SD7          ALT2   ALT3   ALT4   UART1_RXD ) ], # GPIO 15
+    
+    [qw( ALT0          ALT1         ALT2   ALT3   ALT4        ALT5 ) ], # GPIO 16
+    [qw( ALT0          SD9          ALT2   UART0_RTS   SPI1_CE1_N  UART1_RTS ) ], # GPIO 17
+    [qw( PCM_CLK       SD10         ALT2   BSCSL_SDA/MOSI   SPI1_CE0_N   PWM0 ) ], # GPIO 18
+    [qw( ALT0          ALT1         ALT2   ALT3   ALT4        ALT5 ) ], # GPIO 19
+    
+    [qw( ALT0          ALT1         ALT2   ALT3   ALT4        ALT5 ) ], # GPIO 20
+    [qw( ALT0          ALT1         ALT2   ALT3   ALT4        GPCLK1 ) ], # GPIO 21
+    [qw( ALT0          SD14         ALT2   SD1_CLK   ARM_TRST ALT5 ) ], # GPIO 22
+    [qw( ALT0          SD15         ALT2   SD1_CMD   ARM_RTCK ALT5 ) ], # GPIO 23
+    
+    [qw( ALT0          SD16         ALT2   SD1_DAT0  ARM_TDO  ALT5 ) ], # GPIO 24
+    [qw( ALT0          SD17         ALT2   SD1_DAT1  ARM_TCK  ALT5 ) ], # GPIO 25
+    [qw( ALT0          ALT1         ALT2   ALT3   ALT4        ALT5 ) ], # GPIO 26
+    [qw( ALT0          ALT1         ALT2   SD1_DAT3  ARM_TMS  GPCLK1 ) ], # GPIO 27
+    
+    [qw( I2C0_SDA      SA5          PCM_CLK  ALT3   ALT4      ALT5 ) ], # GPIO 28
+    [qw( I2C0_SCL      SA4          PCM_FS   ALT3   ALT4      ALT5 ) ], # GPIO 29
+    [qw( ALT0          SA3          PCM_DIN  UART0_CTS   ALT4 UART1_CTS ) ], # GPIO 30
+    [qw( ALT0          SA2          PCM_DOUT UART0_RTS   ALT4 UART1_RTS ) ], # GPIO 31
+    
+);
+
+
+sub hipi_get_pin {
+    HiPi::BCM2835::Pin->_open( pinid => $_[0] );
+}
+
+sub hipi_gpio_fget_name {
+    my($pinid) = @_;
+    return 'UNKNOWN' if $pinid < 0 || $pinid > 31;
+    given ( hipi_gpio_fget( $pinid )  ) {
+        when(  [ BCM2835_GPIO_FSEL_INPT ] ) {
+            return 'INPUT';
+        }
+        when(  [ BCM2835_GPIO_FSEL_OUTP ] ) {
+            return 'OUTPUT';
+        }
+        when(  [ BCM2835_GPIO_FSEL_ALT0 ] ) {
+            return $_altnames[$pinid]->[0];
+        }
+        when(  [ BCM2835_GPIO_FSEL_ALT1 ] ) {
+            return $_altnames[$pinid]->[1];
+        }
+        when(  [ BCM2835_GPIO_FSEL_ALT2 ] ) {
+            return $_altnames[$pinid]->[2];
+        }
+        when(  [ BCM2835_GPIO_FSEL_ALT3 ] ) {
+            return $_altnames[$pinid]->[3];
+        }
+        when(  [ BCM2835_GPIO_FSEL_ALT4 ] ) {
+            return $_altnames[$pinid]->[4];
+        }
+        when(  [ BCM2835_GPIO_FSEL_ALT5 ] ) {
+            return $_altnames[$pinid]->[5];
+        }
+    }
+}
+
+sub new {
+    my($class, %params) = @_;
+    $params{devicename} = '/dev/mem';
+    my $self = $class->SUPER::new(%params);
+    bcm2835_init();
+    return $self;
+}
+
+sub hipi_set_SPI0 {
+    if($_[0]) {
+        bcm2835_spi_begin();
+    } else {
+        bcm2835_spi_end();
+    }
+}
+
+sub hipi_set_I2C0 {
+    my($on) = @_;
+    
+    my @pins = ( I2C0_SDA,  I2C0_SCL );
+    
+    if( $on ) {
+        for my $pin ( @pins ) {
+            bcm2835_gpio_fsel($pin, BCM2835_GPIO_FSEL_ALT0);
+            bcm2835_gpio_set_pud($pin, BCM2835_GPIO_PUD_UP);
+        }
+    } else {
+        for my $pin ( @pins ) {
+            bcm2835_gpio_fsel($pin, BCM2835_GPIO_FSEL_INPT);
+            bcm2835_gpio_set_pud($pin, BCM2835_GPIO_PUD_OFF);
+        }
+    }
+}
+
+sub hipi_set_I2C1 {
+    my($on) = @_;
+    
+    return if RPI_BOARD_REVISION == 1;
+    
+    my @pins = ( RPI_V2_GPIO_P1_03,  RPI_V2_GPIO_P1_05 );
+    
+    if( $on ) {
+        for my $pin ( @pins ) {
+            bcm2835_gpio_fsel($pin, BCM2835_GPIO_FSEL_ALT0);
+            bcm2835_gpio_set_pud($pin, BCM2835_GPIO_PUD_UP);
+        }
+    } else {
+        for my $pin ( @pins ) {
+            bcm2835_gpio_fsel($pin, BCM2835_GPIO_FSEL_INPT);
+            bcm2835_gpio_set_pud($pin, BCM2835_GPIO_PUD_OFF);
+        }
+    }
+}
+
+sub hipi_set_UART0 {
+    my($on) = @_;
+    my @pins = ( RPI_V2_GPIO_P1_08,  RPI_V2_GPIO_P1_10 );
+    
+    if( $on ) {
+        for my $pin ( @pins ) {
+            bcm2835_gpio_fsel($pin, BCM2835_GPIO_FSEL_ALT0);
+        }
+    } else {
+        for my $pin ( @pins ) {
+            bcm2835_gpio_fsel($pin, BCM2835_GPIO_FSEL_INPT);
+        }
+    }
+}
+
+sub hipi_set_CTS0 {
+    my($on) = @_;
+    
+    my @pins = ( RPI_V2_GPIO_P5_05,  RPI_V2_GPIO_P5_06 );
+    
+    if( $on ) {
+        for my $pin ( @pins ) {
+            bcm2835_gpio_fsel($pin, BCM2835_GPIO_FSEL_ALT3);
+        }
+    } else {
+        for my $pin ( @pins ) {
+            bcm2835_gpio_fsel($pin, BCM2835_GPIO_FSEL_INPT);
+        }
+    }
+}
+
+sub hipi_set_UART1 {
+    my($on) = @_;
+    my @pins = ( RPI_V2_GPIO_P1_08,  RPI_V2_GPIO_P1_10 );
+    
+    if( $on ) {
+        for my $pin ( @pins ) {
+            bcm2835_gpio_fsel($pin, BCM2835_GPIO_FSEL_ALT5);
+        }
+    } else {
+        for my $pin ( @pins ) {
+            bcm2835_gpio_fsel($pin, BCM2835_GPIO_FSEL_INPT);
+        }
+    }
+}
+
+sub hipi_set_CTS1 {
+    my($on) = @_;
+    
+    my @pins = ( RPI_V2_GPIO_P5_05,  RPI_V2_GPIO_P5_06 );
+    
+    if( $on ) {
+        for my $pin ( @pins ) {
+            bcm2835_gpio_fsel($pin, BCM2835_GPIO_FSEL_ALT5);
+        }
+    } else {
+        for my $pin ( @pins ) {
+            bcm2835_gpio_fsel($pin, BCM2835_GPIO_FSEL_INPT);
+        }
+    }
+}
+
+sub hipi_set_PWM0 {
+    my($on) = @_;
+        
+    if( $on ) {
+        bcm2835_gpio_fsel(RPI_V2_GPIO_P1_12, BCM2835_GPIO_FSEL_ALT5);
+    } else {
+        bcm2835_gpio_fsel(RPI_V2_GPIO_P1_12, BCM2835_GPIO_FSEL_INPT);
+    }
+}
 
 1;
+
+__END__
